/*
    This file is part of the 'yarec' package.
    Copyright (C) 1998  Ralf W. Stephan <stephan@tmt.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "soundHandler.h"

#ifndef SOUND_LINUX
#ifndef SOUND_SUN
#error no sound support!
#endif
#endif

struct func_list
{
	void (*init)(int*, int*, int*, int);
	void (*open_again)();
	long (*play_chunk)(u32*, int);
	long (*rec_chunk)(u32*, int);
	void (*tmp_close)();
	void (*close)();
};

extern void LinuxSound_init (int*, int*, int*, int);
extern void LinuxSound_open_again();
extern long LinuxSound_play_chunk (u32*, int);
extern long LinuxSound_rec_chunk (u32*, int);
extern void LinuxSound_tmp_close();
extern void LinuxSound_close();

static struct func_list func_list_[] = 
{
#ifdef SOUND_LINUX
	{
		&LinuxSound_init,
		&LinuxSound_open_again,
		&LinuxSound_play_chunk,
		&LinuxSound_rec_chunk,
		&LinuxSound_tmp_close,
		&LinuxSound_close,
	},
#endif
#ifdef SOUND_SUN
	{
		0,
		0,
		0,
		0,
		0,
		0,
	},
#endif
};

static struct func_list* current_snd_ = 0;

void SoundHandler_init (int *r, int *b, int *c, int p)
{
  current_snd_ = &(func_list_[0]);
  (current_snd_->init)(r, b, c, p);
}

void SoundHandler_open_again()
{
  if (current_snd_)
  	(current_snd_->open_again)();
}

long SoundHandler_play_chunk (u32* p, int size)
{
  if (!current_snd_) return 0;
  return (current_snd_->play_chunk)(p, size);
}

long SoundHandler_rec_chunk (u32* p, int size)
{
  if (!current_snd_) return 0;
  return (current_snd_->rec_chunk)(p, size);
}

void SoundHandler_tmp_close()
{
  if (current_snd_)
  	(current_snd_->tmp_close)();
}

void SoundHandler_close()
{
  if (current_snd_)
  	(current_snd_->close)();
}
